/**
 * 预览效果的本地服务及子应用打包webpack的配置
 * @author yanrong.tian
 * @date 2020/5/16 14:22:00
 */

const path = require('path');
const fs = require('fs');
const lessLoader = require.resolve('less-loader');
const webpack = require('webpack');
const webpackDevMiddleware = require('webpack-dev-middleware');
const ModuleFederationPlugin = require('webpack/lib/container/ModuleFederationPlugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
const Webpack = require('webpack');
const nodeExternals = require('webpack-node-externals');
const MonacoWebpackPlugin = require('monaco-editor-webpack-plugin');
const {alias} = require('./alias');

const { getBabelLoaderConfig, getStyleLoaders } = require('./common');
const lessModuleRegex = /\.module\.(scss|less)$/;

exports.Webpack = Webpack;

exports.webpackDevMiddleware = webpackDevMiddleware;

exports.getWebpackConfig = ({
  layoutRoot,
  publicPath,
  webpackConfig,
  outputPath,
  lessVariables = {},
  entry, // 入口
  requirePath, // 发送请求的文件路径, 控制请求拦载
  command,
  isBundleAnalyzerPlugin,
}) => {
  let config = {
    mode: 'production',

    entry: entry,

    devtool: false,

    externals: [nodeExternals()],

    // experiments: {
    //   outputModule: true,
    //   syncWebAssembly: true,
    //   topLevelAwait: true,
    //   asyncWebAssembly: true,
    //   layers: true,
    // },

    output: {
      path: outputPath,
      filename: '[name].js',
      library: {
        type: 'system',
      }

      //libraryTarget: 'commonjs2',
    },

    resolve: {
      extensions: ['.js', '.ts', '.jsx', '.tsx', '.json', '.mdx'],
    },

    optimization: {
      minimizer: [],
    },

    module: {
      rules: [
        {
          test: [
            /\.bmp$/,
            /\.gif$/,
            /\.jpe?g$/,
            /\.png$/,
            /\.webp$/,
            /\.svg/,
            /\.ttf/,
            /\.woff/,
            /\.eot/,
            /\.otf/,
            /\.woff2/,
          ],
          loader: require.resolve('url-loader'),
          options: {
            limit: 10000,
            name: '[name].[ext]',
            esModule: false,
          },
        },
        {
          test: /\.(ts|tsx|js|jsx|mjs)$/,
          use: [getBabelLoaderConfig()],
        },
        {
          test: /\.css$/, // 解析css
          use: getStyleLoaders(false, {
            importLoaders: 1,
          }),
        },
        {
          test: /\.less$/, // 解析less
          exclude: lessModuleRegex,
          use: getStyleLoaders(
            false,
            {
              importLoaders: 1,
            },
            {
              loader: lessLoader,
              options: {
                lessOptions: {
                  modifyVars: lessVariables,
                  javascriptEnabled: true,
                }

              },
            },
          ),
        },
        {
          test: lessModuleRegex,
          use: getStyleLoaders(
            false,
            {
              importLoaders: 2,
              modules: true,
              localIdentName: '[path][name]__[local]',
              namedExport: true,
              camelCase: true,
            },
            {
              loader: lessLoader,
              options: {
                lessOptions: {
                  modifyVars: lessVariables,
                  javascriptEnabled: true,
                },
              },
            },
          ),
        },
      ],
    },

    plugins: [
      new webpack.DefinePlugin({
        _PROCESS_ENV_PUNLIC_PATH_: '"' + publicPath + '"',
        _PROCESS_ENV_BASE_: '"' + publicPath + '"',
        NODE_ENV: 'production',
      }),

      new CleanWebpackPlugin(),
    ],
  };

  if (isBundleAnalyzerPlugin) {
    config.plugins.push(new BundleAnalyzerPlugin());
  }

  config = webpackConfig(Object.assign({
    devServer: {}
  }, config), {
    requestPath: requirePath,
    moduleFederationPlugin: ModuleFederationPlugin,
    monacoWebpackPlugin: MonacoWebpackPlugin,
    offlinePlugin: function() {},
    command,

  });

  if(config.devServer) {
    delete devServer;
  }

  config.resolve.alias = Object.assign({
    '@layout': layoutRoot
  }, alias, (config.resolve && config.resolve.alias));

  return config;
};
